﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Hive.Slave.Communication.SlaveService;
using HeuristicLab.Hive.Contracts;
using System.ServiceModel;
using HeuristicLab.Hive.Slave.Communication.Properties;
using HeuristicLab.Tracing;

namespace HeuristicLab.Hive.Slave.Communication {
  internal class SlaveFacadeServicePool {
    /// <summary>
    /// Cache of SlaveFacadeClients, to check if all get disposed
    /// </summary>
    private SlaveFacadeClient cachedSlaveFacadeClientStreamed = null;
    private SlaveFacadeClient cachedSlaveFacadeClient = null;
    private int streamedClientCount = 0;
    private int clientCount = 0;
    private string serverIp;
    private static object locker = new object();

    public SlaveFacadeServicePool(string serverIp) {
      this.serverIp = serverIp;
    }

    internal ISlaveFacade CreateSlaveFacade() {
      lock (locker) {
        if (cachedSlaveFacadeClient == null) {
          cachedSlaveFacadeClient = new SlaveFacadeClient("SlaveHttpEndpoint");
          WcfSettings.SetEndpointAddress(cachedSlaveFacadeClient.Endpoint, this.serverIp);
          SetCredentials(cachedSlaveFacadeClient);
          cachedSlaveFacadeClient.Open();
          Logger.Debug("Created SlaveFacadeClient");
        }
        clientCount++;
        Logger.Debug("SlaveFacadeClientCount: " + clientCount);
        return cachedSlaveFacadeClient;
      }
    }

    internal ISlaveFacade CreateStreamedSlaveFacade() {
      lock (locker) {
        if (cachedSlaveFacadeClientStreamed == null) {
          cachedSlaveFacadeClientStreamed = new SlaveFacadeClient("SlaveTcpStreamedEndpoint");
          WcfSettings.SetEndpointAddress(cachedSlaveFacadeClientStreamed.Endpoint, this.serverIp);
          SetCredentials(cachedSlaveFacadeClientStreamed);
          cachedSlaveFacadeClientStreamed.Open();
          Logger.Debug("Created SlaveFacadeClientStreamed");
        }
        streamedClientCount++;
        Logger.Debug("SlaveFacadeClientCount: " + streamedClientCount);
        return cachedSlaveFacadeClientStreamed;
      }
    }

    private void SetCredentials(SlaveFacadeClient client) {
      client.ClientCredentials.UserName.UserName = Settings.Default.HiveUsername;
      client.ClientCredentials.UserName.Password = Settings.Default.HivePassword;
    }

    public void DisposeSlaveClient(ISlaveFacade slaveFacade) {
      lock (locker) {
        if (slaveFacade == cachedSlaveFacadeClient) {
          clientCount--;
          Logger.Debug("Disposing SlaveFacadeClient. Count: " + clientCount);
          if (clientCount < 0)
            throw new Exception("SlaveFacadeClient Count is less than zero. It was released too often.");
          if (clientCount == 0) {
            WcfSettings.DisposeWcfClient((ICommunicationObject)slaveFacade);
            cachedSlaveFacadeClient = null;
          }
        } else if (slaveFacade == cachedSlaveFacadeClientStreamed) {
          streamedClientCount--;
          Logger.Debug("Disposing SlaveFacadeClientStreamed. Count: " + streamedClientCount);
          if (streamedClientCount < 0)
            throw new Exception("SlaveFacadeClientStreamed Count is less than zero. It was released too often.");
          if (streamedClientCount == 0) {
            WcfSettings.DisposeWcfClient((ICommunicationObject)slaveFacade);
            cachedSlaveFacadeClientStreamed = null;
          }
        } else {
          throw new Exception("Cannot dispose slaveFacade. Cannot be found in cache.");
        }
      } // lock
    }
  }
}
